//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Play_Library, Example2 - Connections
//
// Date: 13th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
// Once you have created a DirectPlay interface, you need to setup a DirectPlay
// connection. Connections can be made via IPX, TCP/IP, Modem, Serial-Cable.
//
// This example shows how to enumerate DirectPlay connections are available
// using a created DirectPlay interface. It shows create a DirectPlay connection
// and destroy a DirectPlay connection.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 or 4 only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Play"
#pragma link "TDPName"
#pragma link "TDx_Play_Library_Install"
//---------------------------------------------------------------------------
#pragma link "tdpcaps"
#pragma link "tdpsessiondesc"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
EnumeratingSessions = false;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
if (Dx_Play1) if (Dx_Play1->Created) Dx_Play1->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CreateButtonClick(TObject *Sender)
{
Msg("----");
Msg("Creating DirectPlay Interface");
Dx_Play1->UseAnsiString = true;
Dx_Play1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyButtonClick(TObject *Sender)
{
Msg("----");
Msg("Destroying DirectPlay Interface");
Dx_Play1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Create(TObject *Sender)
{
Msg("----");
Msg("DirectPlay Interface has been Created");
StatusLabel->Caption = "DirectPlay interface is currently created";
StatusLabel->Font->Color = clGreen;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Destroy(TObject *Sender)
{
Msg("----");
Msg("DirectPlay Interface is being Destroyed");

while (NumSessions>0)
	{
    if (Sessions[NumSessions-1]) delete Sessions[NumSessions-1];
    NumSessions--;
    }
while (NumConnections>0)
	{
	if (ConnectionNames[NumConnections-1]) delete ConnectionNames[NumConnections-1];
	if (ConnectionBuffers[NumConnections-1]) delete ConnectionBuffers[NumConnections-1];
    NumConnections--;
    }

ConnectionInitialized = false;

CapsListBox->Items->Clear();
SessionsListBox->Items->Clear();

StatusLabel->Caption = "- DirectPlay interface is not created -";
StatusLabel->Font->Color = clRed;
NumConnectionsLabel->Caption = "- no connections -";
NumConnectionsLabel->Font->Color = clRed;
ConnectionInitializedLabel->Caption = "- not connected -";
ConnectionInitializedLabel->Font->Color = clRed;
NumSessionsLabel->Caption = "- no sessions -";
NumSessionsLabel->Font->Color = clRed;
SessionStatusLabel->Caption = "- not created or joined -";
SessionStatusLabel->Font->Color = clRed;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Error(TObject *Sender, AnsiString Function,
	AnsiString Error, AnsiString ErrorMessage)
{
if ((int)Dx_Play1->ErrorValue==-2005466786) return;	// unknown error: this is returned at runtime when DirectPlay is waiting to initialize a Serial-Cable connection.
if ((int)Dx_Play1->ErrorValue==-2147024891) return; // unknown error: this is returned at runtime when cannot connect via a Serial-Cable connection.

if ((int)Dx_Play1->ErrorValue==-2147467259)	// unknown error: this is returned at runtime when attempting to enumerate sessions after creating a session. 
	{
    SessionsListBox->Items->Clear();
    SessionsListBox->Items->Add( "Cannot enumerate because this" );
    SessionsListBox->Items->Add( "application created the session." );
    return;
    }

// normal error handling
Msg("----");
Msg("An Error Occurred in the DirectPlay Interface");
Msg("Function: "+Function);
Msg("Error: "+Error);
Msg("Error Code: "+IntToStr((int)Dx_Play1->ErrorValue));
Msg("Error Message: "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumConnectionsButtonClick(TObject *Sender)
{
Msg("----");
Msg("Enumerating Connections");

ConnectionsListBox->Items->Clear();
NumConnectionsLabel->Caption = "- no connections -";
NumConnectionsLabel->Font->Color = clRed;

// Enumerate available connections
NumConnections = 0;
Dx_Play1->EnumConnections( NULL, NULL, 0 );

if (NumConnections>0)
	{
    NumConnectionsLabel->Font->Color = clGreen;
	NumConnectionsLabel->Caption = IntToStr((int) NumConnections) + " connections";
    }
else
	{
	NumConnectionsLabel->Font->Color = clRed;
    NumConnectionsLabel->Caption = "- no connections -";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitConnectionButtonClick(TObject *Sender)
{
if (ConnectionInitialized) return;
if (ConnectionsListBox->ItemIndex==-1) return;

Msg("----");
Msg("Initializing Connection");

ConnectionInitialized = Dx_Play1->InitializeConnection( ConnectionBuffers[ ConnectionsListBox->ItemIndex ] );
if (ConnectionInitialized)
	{
	Msg("Connection Established");
	ConnectionInitializedLabel->Caption = "connection established";
    ConnectionInitializedLabel->Font->Color = clGreen;

    CapsListBox->Items->Clear();
    if (Dx_Play1->GetCaps( DPCaps1, 0 ))
        {
        CapsListBox->Items->Add( "Flags= ");
        CapsListBox->Items->AddStrings( DPCaps1->Flags_Strings );
        CapsListBox->Items->Add( "HeaderLength= "+IntToStr( (int) DPCaps1->HeaderLength ) );
        CapsListBox->Items->Add( "HundredBaud= "+IntToStr( (int) DPCaps1->HundredBaud ) );
        CapsListBox->Items->Add( "Latency= "+IntToStr( (int) DPCaps1->Latency ) );
        CapsListBox->Items->Add( "MaxBufferSize= "+IntToStr( (int) DPCaps1->MaxBufferSize ) );
        CapsListBox->Items->Add( "MaxLocalPlayer= "+IntToStr( (int) DPCaps1->MaxLocalPlayers ) );
        CapsListBox->Items->Add( "MaxPlayers= "+IntToStr( (int) DPCaps1->MaxPlayers ) );
        CapsListBox->Items->Add( "Tag= "+IntToStr( (int) DPCaps1->Tag ) );
    	}
	else
    	CapsListBox->Items->Add( "ERROR: Could not retrieve caps" );

	SessionTimer->Enabled = true;
    }
else
	{
	Msg("Connection could not be established");

	ConnectionInitializedLabel->Caption = "- not connected -";
    ConnectionInitializedLabel->Font->Color = clRed;

    CapsListBox->Items->Clear();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumConnections(TObject *Sender,
	const GUID *Guid, void *Connection, DWORD ConnectionSize, TDPName *Name,
	DWORD Flags, void *Context, bool &Continue)
{
if (NumConnections<MAX_CONNECTIONS)
    {
	Msg("Found Connection: " + Name->ShortNameA );
    // save information for later use
    ConnectionGuids[NumConnections] = *Guid;
    ConnectionBuffers[NumConnections] = new byte[ConnectionSize];
    if (ConnectionBuffers[NumConnections])
       CopyMemory( ConnectionBuffers[NumConnections], Connection, ConnectionSize );

    ConnectionSizes[NumConnections] = ConnectionSize;
    ConnectionNames[NumConnections] = new TDPName(this);
    if (ConnectionNames[NumConnections])
    	{
    	ConnectionNames[NumConnections]->LongNameA = Name->LongNameA;
    	ConnectionNames[NumConnections]->ShortNameA = Name->ShortNameA;
        }
    ConnectionFlags[NumConnections] = Flags;

    // update the form's display
	ConnectionsListBox->Items->Add( IntToStr((int) NumConnections)+ ": "+Name->ShortNameA );
	NumConnections++;
	}
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CloseConnectionButtonClick(TObject *Sender)
{
if (ConnectionInitialized)
	{
	Msg("----");
	Msg("Closing Connection");
    
	// simply destroy the Dx_Play component to close the connection
	if (Dx_Play1->Created) Dx_Play1->Destroy();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SessionTimerTimer(TObject *Sender)
{
if (!Dx_Play1->Created) return;
if (EnumeratingSessions) return;

Msg("----");
Msg("Scanning for available Sessions");

SessionsListBox->Items->Clear();
NumSessions = 0;
DPSessionDesc1->Clear();
EnumeratingSessions = true;
Dx_Play1->EnumSessions( DPSessionDesc1, 0, NULL, DPENUMSESSIONS_ALL );
EnumeratingSessions = false;
if (NumSessions>0)
	{
	Msg("Found "+IntToStr((int)NumSessions)+" Sessions");

    NumSessionsLabel->Caption = IntToStr((int)NumSessions) + " Sessions";
    NumSessionsLabel->Font->Color = clGreen;
    SessionTimer->Enabled = false;
	SessionOpen = false;
	SessionJoined = false;
	SessionCreated = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSessionButtonClick(TObject *Sender)
{
if (SessionOpen) return;

DPSessionDesc1->SessionNameA = "EXAMPLE 3 - TEST SESSION";
DPSessionDesc1->MaxPlayers = 100;
DPSessionDesc1->Flags = DPSESSION_KEEPALIVE | DPSESSION_MIGRATEHOST | DPSESSION_DIRECTPLAYPROTOCOL;

if (Dx_Play1->Open( DPSessionDesc1, DPOPEN_CREATE ))
    {
    SessionOpen = true;
    SessionJoined = false;
    SessionCreated = true;

    NumSessionsLabel->Caption = "Created current session";
    NumSessionsLabel->Font->Color = clGreen;

    SessionStatusLabel->Caption = DPSessionDesc1->SessionNameA + ", "+IntToStr((int)DPSessionDesc1->MaxPlayers) + " Max Players";
    SessionStatusLabel->Font->Color = clGreen;

    SessionTimer->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::JoinSessionButtonClick(TObject *Sender)
{
if (SessionsListBox->ItemIndex==-1) return;

if (Dx_Play1->Open( Sessions[SessionsListBox->ItemIndex], DPOPEN_JOIN ))
   	{
	SessionOpen = true;
    SessionJoined = true;
    SessionCreated = false;

    NumSessionsLabel->Caption = "Joined current session";
    NumSessionsLabel->Font->Color = clGreen;

	SessionStatusLabel->Caption = Sessions[SessionsListBox->ItemIndex]->SessionNameA;
    SessionStatusLabel->Font->Color = clGreen;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CloseSessionButtonClick(TObject *Sender)
{
if (Dx_Play1->Close())
    {
    SessionTimer->Enabled = false;

    SessionOpen = false;
    SessionJoined = false;
    SessionCreated = false;

	NumSessionsLabel->Caption = "- no sessions -";
	NumSessionsLabel->Font->Color = clRed;

	SessionStatusLabel->Caption = "- not created or joined -";
    SessionStatusLabel->Font->Color = clRed;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumSessions(TObject *Sender,
	TDPSessionDesc *SessionDesc, DWORD *TimeOut, DWORD Flags, void *Context,
	bool &Continue)
{
if (NumSessions<MAX_SESSIONS)
	{
    SessionsListBox->Items->Add( SessionDesc->SessionNameA );

	Sessions[NumSessions] = new TDPSessionDesc(this);
    CopyMemory( Sessions[NumSessions]->Internal_DPSESSIONDESC2_Ptr, SessionDesc->Internal_DPSESSIONDESC2_Ptr, sizeof(DPSESSIONDESC2) );

    NumSessions++;
    }
}
//---------------------------------------------------------------------------
